{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzStpl;         {buzStpl  buzGlobal}

{***********************************************************************
NAME                           STATE PLANE
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the State Plane projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan		Apr, 1993
ALGORITHM REFERENCES
1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/
}

interface

uses Math,Sysutils;
var
id: integer;
inzone: integer = 0;

function  stplnforint(zone, sphere: Integer):     Integer;

function  stplnfor(lon, lat: double; var x, y: double): Integer;

function  stplninvint(zone, sphere: Integer ):    Integer;

function  stplninv(x, y: double; var lon, lat: double): Integer;

implementation

uses buzGlobal, buzTm, buzLamcc, buzPoly, buzOmer;

// Initialize the State Plane projection
function  stplnforint(zone, sphere: Integer ): Integer;
var
ind,i, nadval,mode,iflg:Integer;
table: array[0..8]of Double;
pname:String[32];
 r_maj,r_min,scale_fact,center_lon,
 center_lat,false_east,false_north,
 azimuth,lat_orig,lon_orig,lon1,lat1,lon2,lat2:double;
 ptr:String;
 SPFile:FILE ;
{char buf[100];
double paksz(double ang, long int *iflg);
double pakcz(double pak);}
begin
{  Result := 1;}
  ind := -1;
  iflg:=0;
  if (zone > 0)then
    Begin
      if (sphere = 0)then
      begin
        for i := 0 to 133 do
        begin
          if (zone = NAD27[i])then
          begin
            ind := i;
            break;
          end;
      end;
     end
   else
   if (sphere = 8) then { 8: GRS 1980}
   begin
      for i := 0 to 133 do
      begin
         if (zone = NAD83[i])then
     begin
            ind := i;
            break;
            end;
         end;
      end;
   end;
   if (ind = -1)then
   begin
      { sprintf(buf,"Illegal zone #%4d  for spheroid #%4d",zone,sphere);
       p_error(buf,"state-init");}
     DoProjError(21,21,zone,sphere,0);
     Result:=(21);
     Exit;
   end;

if (inzone <> zone)then
begin
    inzone := zone;
    if (sphere = 0)then
    begin
       ptr := ExtractFilePath(ParamStr(0))+'nad27sp';
    end
    else
    begin
       ptr := ExtractFilePath(ParamStr(0))+'nad83sp';
    end;

    if (not FileExists(ptr))then
    begin
       {p_error("Error opening State Plane parameter file","state-for");}
      DoProjError(21,22,0,0,0);
      Result:=(22);
        Exit;
    end else
    begin
      AssignFile(SPFile, ptr);
      Reset(SPFile,1);
      if IoResult <> 0 then DoProjError(7,7,0,0,0);{DoRezError(22);}
    seek(SPFile,
    (ind * (
    ( sizeof(char) * 32) +
    sizeof(Integer) +
    (sizeof(double) * 9) ) ));
    BlockRead(SPFile,pname,sizeof(char)*32);
    BlockRead(SPFile,id,sizeof(Integer));
    BlockRead(SPFile,table,sizeof(double)*9);
    CloseFile(SPFile);
    end;
    if (id <= 0)then
    begin
       { sprintf(buf,"Illegal zone #%4d  for spheroid #%4d",zone,sphere);
        p_error(buf,"state-init");}
      DoProjError(21,21,zone,sphere,0);
      Result:=(21);
      Exit;
    end;

    // Report parameters to the user
    ptitle('STATE PLANE');
    GenrptLong(zone,'Zone:     ');
    if (sphere = 0)then nadval := 27
    else nadval := 83;
    GenrptLong(nadval,'Datum:     NAD');
end;

r_maj := table[0];
r_min := (sqrt(1.0 - table[1])) * table[0];

// initialize proper projection
if (id = 1)then
   begin
   scale_fact := table[3];
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0) then begin Result:=(iflg); exit;end;
   center_lat := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then  begin Result:=(iflg); exit;end;
   false_east := table[7];
   false_north := table[8];
   tmforint(r_maj,r_min,scale_fact,center_lon,center_lat,
            false_east, false_north);
  end
else
if (id = 2)then
   begin
   lat1 := paksz(pakcz(table[5]),iflg) * D2R;
   if (iflg <> 0)then  begin Result:=(iflg); exit;end;
   lat2 := paksz(pakcz(table[4]),iflg) * D2R;
   if (iflg <> 0)then  begin Result:=(iflg); exit;end;
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then  begin Result:=(iflg);exit;end;
   center_lat := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then  begin Result:=(iflg); exit;end;
   false_east := table[7];
   false_north := table[8];
   lamccforint(r_maj,r_min,lat1,lat2,center_lon,center_lat,
               false_east,false_north);
   end
else
if (id = 3)then
   begin
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then   begin Result:=(iflg); exit;end;
   center_lat := paksz(pakcz(table[3]),iflg) * D2R;
   if (iflg <> 0)then   begin Result:=(iflg); exit;end;
   false_east := table[4];
   false_north := table[5];
   polyforint(r_maj,r_min,center_lon,center_lat,
              false_east,false_north);
   end
else
if (id = 4)then
   begin
   scale_fact := table[3];
   azimuth := paksz(pakcz(table[5]),iflg) * D2R;
   if (iflg <> 0) then  begin  Result:=(iflg);exit;end;
   lon_orig := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then   begin  Result:=(iflg);exit;end;
   lat_orig := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then   begin  Result:=(iflg);exit;end;
   false_east := table[7];
   false_north := table[8];
   mode := 1;
   lon1:=inparm[8];{Shut up compiler: mode 1 ..these NA}
   lat1:=inparm[9];
   lon2:=inparm[10];
   lat2:=inparm[11];
   omerforint(r_maj,r_min,scale_fact,azimuth,lon_orig,lat_orig,
              false_east,false_north,lon1,lat1,lon2,lat2,mode);
   end;
Result:=(OK);
End;


// State plane forward equations--mapping lat,long to x,y
function  stplnfor(lon, lat: double; var x, y: double): Integer;
              // (I) Longitude
              // (I) Latitude
             // (O) X projection coordinate
             // (O) Y projection coordinate
var iflg:Integer;
begin
// Forward equations
if (id = 1)then
   begin  iflg := tmfor(lon, lat, x, y);
   if ( iflg <> 0)then  begin  Result := (iflg);exit;end;
   End
else
if (id = 2)then
   begin  iflg := lamccfor(lon, lat, x, y);
   if (iflg <> 0)then  begin  Result := (iflg);exit;end;
   End
else
if (id = 3)then
   begin  iflg := polyfor(lon, lat, x, y);
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   End
else
if (id = 4)then
   begin  iflg := omerfor(lon, lat, x, y);
   if (iflg <> 0)then  begin Result := (iflg); exit;end;
   End;
Result := (OK);
End;


// Initialize the State Plane projection
{
long ind;
long i;
long nadval;
static double table[9];
static char pname[33];
char buf[100];
double paksz(double ang, long int *iflg);
double pakcz(double pak);
double r_maj,r_min,scale_fact,center_lon;
double center_lat,false_east,false_north;
double azimuth,lat_orig,lon_orig,lon1,lat1,lon2,lat2;
long mode,iflg;
long status;
FILE *ptr;}
function  stplninvint(zone, sphere: Integer):    Integer;
var
ind,i, nadval,mode,iflg:Integer;
table: array[0..8]of Double;
pname:String[32];
 r_maj,r_min,scale_fact,center_lon,
 center_lat,false_east,false_north,
 azimuth,lat_orig,lon_orig,lon1,lat1,lon2,lat2:double;
 ptr:String;
 SPFile:FILE ;
begin
ind := -1;
iflg := -1;{Compiler upshut}
if (zone > 0)then
begin
    if (sphere = 0)then
    begin
        for i := 0 to 133 do
        begin
         if (zone = nad27[i]) then
         begin
                ind := i;
                break;
         End;
     End;
    End
    else if (sphere = 8)then
    begin
        for i := 0 to 133 do
        begin
            if (zone = nad83[i]) then
         begin
                ind := i;
                break;
            End;
        End;
    End;
End;

if (ind = -1)then
begin
    {sprintf(buf,"Illegal zone #%4d  for spheroid #%4d",zone,sphere);
    p_error(buf,"state-init");   }
       DoProjError(21,21,zone,sphere,0);
    Result :=(21);
    Exit;
End;

if (inzone <> zone)then
begin
    inzone := zone;
    if (sphere = 0)then
 begin
       ptr := ExtractFilePath(ParamStr(0))+'nad27sp';
    end
    else
    begin
       ptr := ExtractFilePath(ParamStr(0))+'nad83sp';
    end;

    if (not FileExists(ptr))then
    begin
       {p_error("Error opening State Plane parameter file","state-for");}
      DoProjError(21,22,0,0,0);
      Result:=(22);
        Exit;
    end else
    begin
      AssignFile(SPFile, ptr);
      Reset(SPFile,1);
      if IoResult <> 0 then DoProjError(7,7,0,0,0);{DoRezError(22);}
    seek(SPFile,
    (ind * (
    ( sizeof(char) * 32) +
    sizeof(Integer) +
    (sizeof(double) * 9) ) ));
    BlockRead(SPFile,pname,sizeof(char)*32);
    BlockRead(SPFile,id,sizeof(Integer));
    BlockRead(SPFile,table,sizeof(double)*9);
    CloseFile(SPFile);
    end;

    if (id <= 0)then
    begin
        {sprintf(buf,"Illegal zone #%4d  for spheroid #%4d",zone,sphere);
        p_error(buf,"state-init"); }
           DoProjError(21,21,zone,sphere,0);
        Result :=(21);
        Exit;
    End;

    // Report parameters to the user
    ptitle('STATE PLANE');
    genrptlong(zone,'Zone:     ');
    if (sphere = 0) then       nadval := 27
    else       nadval := 83;
    genrptlong(nadval,'Datum:     NAD');
End;

// initialize proper projection
r_maj := table[0];
r_min := (sqrt(1.0 - table[1])) * table[0];

if (id = 1)then
   begin
   scale_fact := table[3];
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   center_lat := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   false_east := table[7];
   false_north := table[8];
   tminvint(r_maj,r_min,scale_fact,center_lon,center_lat,false_east,
     false_north);
   End
else
if (id = 2)then
   begin
   lat1 := paksz(pakcz(table[5]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   lat2 := paksz(pakcz(table[4]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   center_lat := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   false_east := table[7];
   false_north := table[8];
   lamccinvint(r_maj,r_min,lat1,lat2,center_lon,center_lat,false_east,
        false_north);
   End
else
if (id = 3)then
   begin
   center_lon := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   center_lat := paksz(pakcz(table[3]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   false_east := table[4];
   false_north := table[5];
   polyinvint(r_maj,r_min,center_lon,center_lat,false_east,false_north);
   End
else
if (id = 4)then
   begin
   scale_fact := table[3];
   azimuth := paksz(pakcz(table[5]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   lon_orig := paksz(pakcz(table[2]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   lat_orig := paksz(pakcz(table[6]),iflg) * D2R;
   if (iflg <> 0)then  begin Result := (iflg);exit;end;
   false_east := table[7];
   false_north := table[8];
   mode := 1;
   lon1:=inparm[8];{Shut up compiler: mode 1 ..these NA}
   lat1:=inparm[9];
   lon2:=inparm[10];
   lat2:=inparm[11];
   omerinvint(r_maj,r_min,scale_fact,azimuth,lon_orig,lat_orig,false_east,
              false_north,lon1,lat1,lon2,lat2,mode);
   End;
Result :=(OK);
End;


function
  stplninv(x, y: double; var lon, lat: double): Integer;
{State Plane inverse equations--mapping x,y to lat/long
long stplninv(
double x,/* (O) X projection coordinate 	*/
double y,/* (O) Y projection coordinate 	*/
 double *lon,/* (I) Longitude 		*/
 double *lat)/* (I) Latitude 		*/}
var iflg: Integer;
begin
  result := (OK);
//* Inverse equations  -----------------*/
  if (id = 1) then
  begin
    iflg := tminv(x, y, lon, lat);
    if (iflg <> 0) then result := (iflg);
  end
  else
    if (id = 2) then
    begin
      iflg := lamccinv(x, y, lon, lat);
      if (iflg <> 0) then result := (iflg);
    end
    else
      if (id = 3) then
      begin
        iflg := polyinv(x, y, lon, lat);
        if (iflg <> 0) then result := (iflg);
      end
      else
        if (id = 4) then
        begin
          iflg := omerinv(x, y, lon, lat);
          if (iflg <> 0) then result := (iflg);
        end;
end;



end.
